* Paddy Carter and Nicolas Van de Sijpe, 11-July-2018
* Define program elusive used to run Monte Carlo simulations


* This do file is called by MCanalysis.do so we have commented out the preamble 
* The preamble is useful to do a single run to check the code works 
* If you want to run the program interactively to learn how it works, it's probably worth saving a version of this and getting rid of all the /* */ which can't be used interactively, setting parameters as locals etc. 


/*
clear all
set more off
set seed 7542629 // seed for a single run of the code below, to see whether program works



*************************************************************************
* GTOOLS	
*************************************************************************
/*
* We use gtools (version 0.13.1 02May2018) to speed up the MCs run by elusive: https://github.com/mcaceresb/stata-gtools
* To install:
local github "https://raw.githubusercontent.com"
net install gtools, from(`github'/mcaceresb/stata-gtools/master/build/)
* Note for Windows users: It may be necessary to run 
gtools, dependencies 
* at the start of your Stata session.
*/


*************************************************************************
*************************************************************************
* MONTE CARLO PROGRAM	
*************************************************************************
*************************************************************************
/* Program key for program elusive below: */
/* (nC num countries) (t years) (nD num DFIs) (phi DFI country preferences weight) (drift in budget random walk as % of initial budget db) 
(sig s.d. of dfi budget as % of initial budget db) (dfbudg t=1 DFI budget % of total ops) (tvweight weight on time-varying part of DFI preferences) 
(mu mean project characteristics of three country types) (musd s.d. for project characteristics of three country types) (ersd s.d. of expected returns added to pc)
(dfilo lower bound eligible dfi set) (dfihi upper bound dfi set) (psmin min exp return for private sector) 
(nI num inv opportunities per country-year) (noise error added to pc after collapse) estimation(estimator options are OLSFE, GMM, IV) 
(mechanism DFI inv selection mechanism: options are random, pclow)*/


capture program drop elusive
program define elusive, rclass
	version 15.1
	syntax [, nC(integer 80) t(integer 20) nD(real 3) phi(real 1) drift(real 0) sig(real 0.2) /// 
	dfbudg(real 0.1) tvweight(real 0) mu1(real 0) mu2(real 2) mu3(real 4) musd1(real 1) musd2(real 1) musd3(real 1) ersd(real 1) /// 
	dfilo(real 2) dfihi(real 8) psmin(real 2) nI(real 50) noise(real 0.5) estimator(string) mechanism(string)]

	
drop _all


/* Locals for single run of the program; otherwise commented out
local nC = 80
local t = 20
local nD = 3
local phi = 1
local drift = 0
local sig = 0.2
local dfbudg = 0.1
local tvweight = 0
local mu1 = 0
local mu2 = 2
local mu3 = 4
local musd1 = 1
local musd2 = 1
local musd3 = 1
local ersd = 1
local dfilo = 2
local dfihi = 8
local psmin = 2
local nI = 50
local noise = 0.5
local estimator OLSFE
local mechanism random
*/


* private sector budget (always exceeds available investment opportunities): 
local pb = `nI'*`nC'
* initial value for the total DFI budget in year 1; defined as a fraction (dfbudg) of maximum potential total number of annual investments:
local db = `dfbudg'*(`nI'*`nC')
* standard deviation for the random walk in the DFI budget process (defined as fraction sig of initial budget db):
local sigma = `sig'*`db'


/* Set up data: */	
* dataset:
local size = `nC'*`t'
set obs `size'
* number the countries:
gen country = ceil(_n/`t') 
* periods:
sort country	
by country: generate year = _n
xtset country year


**************************************************************************
* Generate weights to divide investments between multiple DFIs
**************************************************************************
* generate individual DFIs' country preferences that sum to 1 
forvalues j = 1/`nD' {
	sort year country
	gen double dfipref`j' = runiform() in 1/`nC' // generate across countries in first year only
	gegen double temp`j' = max(dfipref`j'), by(country) // this fills in each country's prefs across years
	drop dfipref`j'
	gen double tvtemp`j' = runiform() // time-varying part of DFI-country preferences.
	gen double dfipref`j' = (1-`tvweight')*temp`j' + `tvweight'*tvtemp`j'
	drop temp`j' tvtemp`j'
}
ds dfipref*
gegen double sum = rowtotal(`r(varlist)')
forvalues j = 1/`nD' {
	replace dfipref`j' = dfipref`j'/sum
}
drop sum


* generate individual DFIs' shares of each investment in each country 
* start with weighted preferences: weight = pref^phi
* turn into shares using share_i = weight_i / (sum weights) 
forvalues j = 1/`nD' {
	gen double dfiweight`j' = dfipref`j'^`phi'
}
ds dfiweight*
gegen double sum = rowtotal(`r(varlist)')
forvalues j = 1/`nD' {
	gen double dfishare`j' = dfiweight`j'/sum
}
drop sum


**************************************************************************
* Country types
**************************************************************************
/* Initial country types: */
* Half of the countries start as type 1, 1/3 as type 2, 1/6 as type 3
gen type = 1
replace type = 2 if country>`nC'*0.5
replace type = 3 if country>`nC'*(5/6)


/* Evolution of country types over time: */
* generate a 0-1 random number to base probabilities on (also used elsewhere later on). 
generate double randnum = uniform()
/* We want to implement the following transition matrix: .85 & .10 & .05\\ .05 & .85 & .10\\ .05 & .05 & .9
Run transition matrix to change country types over time: */
sort country year
forvalues j = 2/`t' {
	// type 1 becoming type 2:
	replace type=2 if randnum>0.85 & randnum<=0.95 & L.type==1 & year==`j'
	// type 1 becoming type 3:
	replace type=3 if randnum>0.95 & L.type==1 & year==`j'
	
	// type 2 becoming type 1:
	replace type=1 if randnum<=0.05 & L.type==2 & year==`j'
	// type 2 becoming type 3
	replace type=3 if randnum>0.9 & L.type==2 & year==`j'
	
	// type 3 becoming type 1:
	replace type=1 if randnum<=0.05 & L.type==3 & year==`j'
	// type 3 becoming type 2:
	replace type=2 if randnum>0.05 & randnum<=0.1 & L.type==3 & year==`j'
}


**************************************************************************
* Budgets
**************************************************************************
/* Time-varying DFI budget. */
* a random walk with drift 
sort country year
local normdrift=`drift'*`db'
gen int db = `db' in 1/`t' // this just does the first country
* set budget so that it cannot fall below 20; and round so that db is always integer
replace db = round(max(20,L.db + `normdrift' + rnormal(0,`sigma'))) if year > 1 & db != .
gegen double db2 = max(db), by(year) // this fills in the same db for each year for the other countries
drop db
rename db2 db


/* Private budget. */
* Does not need to be stochastic, all that matters is whether pb is larger than bankable set of investments (as specified at the start of the code)
gen double pb = `pb'


**************************************************************************
* Individual projects
**************************************************************************
/* Data for individual projects */
expand `nI'
sort country year
* create an index for individual projects
gen i = _n


/* Project characteristics */
gen double pc = rnormal(`mu1',`musd1')
replace pc = rnormal(`mu2',`musd2') if type==2
replace pc = rnormal(`mu3',`musd3') if type==3
label var pc "Project characteristics"


/* Expected returns */
gen double er = pc + rnormal(0,`ersd')
label var er "Expected returns"


********************************************************************
* DFI investment selection mechanism
********************************************************************
if "`mechanism'" == "random" {
	/* DFI set of eligible projects is based on expected return. 
	Within this global set, in each year the DFI randomly picks projects until its budget runs out. */


	/* Selection of investments by DFI sector as a whole. */
	* Set of investments DFIs would like to invest in:
	gen dfiset = inrange(er,`dfilo',`dfihi')
	/* In each year, DFI sector picks investments from this global set of investment opportunities (so not within each county) until the budget is exhausted */
	/* Put investable projects in random order and pick from the top until the budget is exhausted, 
	so that DFI sector randomly picks projects from its global set of investment opportunities until it exhaust its budget. */
	replace randnum = uniform()
	sort year randnum // randomly sort projects within each year
	* cumulative count of number of projects in each year DFI is interested in:
	by year: gen cum_dfi = sum(dfiset) 
	* DFI sector invests if project is in eligible set and still within the budget:
	generate byte dfi_inv = dfiset == 1 & cum_dfi <= db 


	* Divide up these investments between multiple DFIs
	forvalues j = 1/`nD' {
		gen double dfi_inv_`j' = dfishare`j'*dfi_inv
	}


	/* Selection of investment by private investors: */
	gen psset = 0
	* DFI sector has first dibs so private sector can only pick from investments not already taken by DFI:
	replace psset = 1 if er > `psmin' & dfi_inv == 0
	* same trick to have private investors pick randomly from global set of projects they are interested in in each year
	sort year randnum
	by year: gen cum_ps = sum(psset)
	* private sector invests if the project is in the eligible set and still within the budget:
	generate byte ps_inv = psset == 1 & cum_ps <= pb

}
else if "`mechanism'" == "pclow" {

	/* DFI set of eligible projects is based on expected return. 
	Within this global set, in each year the DFI now first picks the projects with the worst project characteristics. */


	/* Selection of investments by DFI sector as a whole. */
	* Set of investments DFIs would like to invest in:
	gen dfiset = inrange(er,`dfilo',`dfihi')
	/* In each year, DFI sector picks investments from this global set of investment opportunities (so not within each county) until the budget is exhausted */
	/* Rank investment opportunities within each year by pc and pick lowest pc projects first, until budget is exhausted. */
	sort year pc
	* cumulative count of number of projects in each year DFI is interested in:
	by year: gen cum_dfi = sum(dfiset)
	* DFI invests if project is in eligible set and still within the budget:
	generate byte dfi_inv = dfiset == 1 & cum_dfi <= db


	* Divide up these investments between multiple DFIs
	forvalues j = 1/`nD' {
		gen double dfi_inv_`j' = dfishare`j'*dfi_inv
	}


	/* Selection of investment by private investors: */
	gen psset=0
	* DFI has first dibs so private sector can only pick from investments not already taken by DFI:
	replace psset = 1 if er > `psmin' & dfi_inv == 0
	* have private investors pick randomly from global set of projects they are interested in in each year
	replace randnum = uniform()
	sort year randnum
	by year: gen cum_ps = sum(psset)
	* private sector invests if the project is in the eligible set and still within the budget:
	generate byte ps_inv = psset == 1 & cum_ps <= pb 

}
else {
	di as error "Incorrect investment selection mechanism specified"
}


/*
*************************************************************
* Check percentages of bankable projects in single draw 
*************************************************************
/* Check what percentage of investment opportunities are bankable for the private and DFI sectors. 
This is done in a single run with large M (50000) and with DFI set for er between 0 and 2; afterwards this code is commented out. */
// percentage of projects bankable for private sector and DFIs, and median project characterists by type
foreach num of numlist 1/3 {
	count if type == `num'
	scalar type`num' = r(N)
	count if er > `psmin' & type == `num'
	scalar type`num'bankpriv = r(N)
	count if dfiset == 1 & type == `num'
	scalar type`num'bankDFI = r(N)
	di "Percentage of bankable projects for type `num' for private sector is " (type`num'bankpriv/type`num')*100
	di "Percentage of bankable projects for type `num' for DFIs is " (type`num'bankDFI/type`num')*100
	su pc if type == `num' & dfiset == 1, detail
}
*/


******************************************************************************
* Move to dyadic format
******************************************************************************
/* create different DFIs */
sort country year
expand `nD'
sort i
gegen dfi_id = seq(), f(1) t(`nD')
label var dfi_id "DFI name"
gen double dfiinvestment = 0
label var dfiinvestment "Investments by DFIs"
forvalues j = 1/`nD' {
	replace dfiinvestment = dfi_inv_`j' if dfi_id == `j'
}
order country year dfi_id i


********************************************************************************
* Collapse data to country-year-DFI format by summing over projects
********************************************************************************
sort country year dfi_id i
gcollapse (mean) type db pc (sum) dfiinvestment ps_inv, by(country year dfi_id) 
label var db "DFI budget"
label var pc "Mean project characterstics"
/* WARNING: now ps_inv is there multiple times (once for each DFI)
so do not sum within country year to calculate total private investment for each country-year. */


************************************************************
* Generate supply-push instrument
************************************************************
sort country year dfi_id
sort country dfi_id year
// calculate budget of each DFI in each year. Summed across DFIs, this equals the variable db
bysort dfi_id year (country): gegen double DFIbudgetannual = sum(dfiinvestment)
// calculate each DFI's investments in each country as a share of each DFI's annual budget:
gen double shareRDFIannual = (dfiinvestment/DFIbudgetannual) 
// double check that shares sum to 1 when summed over all recipient countries within year and DFI:
/*
bysort dfi_id year (country): egen double sharecheck = sum(shareRDFIannual)
su sharecheck
drop sharecheck
*/
// calculate initial share by averaging annual shares over the first 5 years:
bysort country dfi_id (year): gegen double shareRDFIinittemp = mean(shareRDFIannual) if year <= 5
bysort country dfi_id (year): gegen double shareRDFIinit = max(shareRDFIinittemp)
drop shareRDFIinittemp
// generate instrument as initial share*total budget
gen double DFIinvIV = shareRDFIinit*DFIbudgetannual
*browse country year dfi_id dfiinvestment DFIbudgetannual db shareRDFIinit DFIinvIV


********************************************************************************
* Collapse data to country-year format
********************************************************************************
sort country year dfi_id
d, fulln
*browse
gcollapse (mean) type pc db ps_inv (sum) dfiinvestment DFIinvIV, by(country year)
 

********************************************************************************
* REGRESSIONS
********************************************************************************
/* Calculate total investment for regressions: */
sort country year
gen double totalinv = dfiinvestment + ps_inv


/* Generate time dummies */
su year
local yearmax = r(max)
foreach num of numlist 1(1)`yearmax' {
	gen ydummy`num'=(year==`num')
}


/* add noise to pc */
replace pc = pc + rnormal(0,`noise')


if "`estimator'" == "OLSFE" {

	/* OLS */
	reg totalinv dfiinvestment ydummy*, cluster(country)
	return scalar olsbeta = _b[dfiinvestment]
	return scalar olsse = _se[dfiinvestment]
	return scalar olsr2 = e(r2)
	test dfiinvestment
	return scalar olspzeroadd = ttail(r(df_r), _b[dfiinvestment]/_se[dfiinvestment]) // one-sided test of null that beta <= 0 against beta > 0 (null is zero additionality)
	return scalar olspfulladd = ttail(r(df_r), -(_b[dfiinvestment]-1)/_se[dfiinvestment]) // one-sided test of null that beta >= 1 against beta < 1 (null is full additionality)
	/* OLS controlling for pc  */
	reg totalinv dfiinvestment pc ydummy*, cluster(country)
	return scalar olspcbeta = _b[dfiinvestment]
	return scalar olspcse = _se[dfiinvestment]
	return scalar olspcr2 = e(r2)
	test dfiinvestment
	return scalar olspcpzeroadd = ttail(r(df_r), _b[dfiinvestment]/_se[dfiinvestment]) 
	return scalar olspcpfulladd = ttail(r(df_r), -(_b[dfiinvestment]-1)/_se[dfiinvestment])
	/* OLS with pc only (to pick up R2, with and without time dummies) */
	reg totalinv pc ydummy*, cluster(country)
	return scalar olspcnodfir2 = e(r2)
	reg totalinv pc, cluster(country)
	return scalar olspcnodfinotdummiesr2 = e(r2)
	
	/* FE */
	xtreg totalinv dfiinvestment ydummy*, fe cluster(country) 
	return scalar febeta = _b[dfiinvestment]
	return scalar fese = _se[dfiinvestment]
	return scalar fewr2 = e(r2_w) 
	test dfiinvestment
	return scalar fepzeroadd = ttail(r(df_r), _b[dfiinvestment]/_se[dfiinvestment]) 
	return scalar fepfulladd = ttail(r(df_r), -(_b[dfiinvestment]-1)/_se[dfiinvestment]) 
	/* FE controlling for pc */
	xtreg totalinv dfiinvestment pc ydummy*, fe cluster(country) 
	return scalar fepcbeta = _b[dfiinvestment]
	return scalar fepcse = _se[dfiinvestment]
	return scalar fepcwr2 = e(r2_w) 
	test dfiinvestment
	return scalar fepcpzeroadd = ttail(r(df_r), _b[dfiinvestment]/_se[dfiinvestment]) 
	return scalar fepcpfulladd = ttail(r(df_r), -(_b[dfiinvestment]-1)/_se[dfiinvestment]) 
	/* FE with pc only (to pick up within R2, with and without time dummies) */
	xtreg totalinv pc ydummy*, fe cluster(country) 
	return scalar fepcnodfiwr2 = e(r2_w) 
	xtreg totalinv pc, fe cluster(country) 
	return scalar fepcnodfinotdummieswr2 = e(r2_w) 

}
else if "`estimator'" == "GMM" {

	/* SYS GMM  */
	gen double ltotalinv = L.totalinv
	xtabond2 totalinv ltotalinv dfiinvestment pc ydummy*, gmm(totalinv, laglimits(2 5) collapse split) /// 
	gmm(dfiinvestment, laglimits(2 5) collapse) gmm(pc, laglimits(1 4) collapse) iv(ydummy*, eq(level)) twostep robust small
	return scalar sysgmmbeta = _b[dfiinvestment]
	return scalar sysgmmse = _se[dfiinvestment]
	return scalar sysgmmlagdep = _b[ltotalinv]
	return scalar sysgmmm1p = e(ar1p) // to "pass", want this to reject < 0.05
	return scalar sysgmmm2p = e(ar2p) // want this to fail to reject > 0.1
	return scalar syshanspval = e(hansenp) // want this to fail to reject > 0.1 (null is instruments valid)
	return scalar syssargpval = e(sarganp) // want this to fail to reject > 0.1 (null is instruments valid)
	matrix define A = e(diffsargan)
	matrix list A
	return scalar diffHp_lev = A[5,1] // all moment conditions related to the equations in levels
	return scalar diffHp_lagdepdiff = A[5,2] // moment conditions for lag dep in differenced equations
	return scalar diffHp_lagdeplevel = A[5,3] // moment conditions for lag dep in levels equations
	return scalar diffHp_DFIinv = A[5,4] // moment conditions related to endogenous DFI investment
	return scalar diffHp_pc = A[5,5] // moment conditions related to predetermined pc
	return scalar diffHp_excllev = A[4,1] // excluding moment conditions related to the equations in levels
	return scalar diffHp_excllagdepdiff = A[4,2] // excluding moment conditions for lag dep in differenced equations
	return scalar diffHp_excllagdeplevel = A[4,3] // excluding moment conditions for lag dep in levels equations
	return scalar diffHp_exclDFIinv = A[4,4] // excluding moment conditions related to endogenous DFI investment
	return scalar diffHp_exclpc = A[4,5] // excluding moment conditions related to exogenous pc
	test dfiinvestment
	return scalar sysgmmpcpzeroadd = ttail(r(df_r), _b[dfiinvestment]/_se[dfiinvestment]) 
	return scalar sysgmmpcpfulladd = ttail(r(df_r), -(_b[dfiinvestment]-1)/_se[dfiinvestment]) 
	local dftest = r(df_r)
	/* Now calculate LR effect and carry out LR additionality tests */
	nlcom LRDFIinv: _b[dfiinvestment]/(1-_b[ltotalinv]), post
	return scalar sysgmmbetaLR =_b[LRDFIinv]
	return scalar sysgmmseLR =_se[LRDFIinv]
	test _b[LRDFIinv] = 0
	return scalar sysgmmpcpzeroaddLR = ttail(`dftest', _b[LRDFIinv]/_se[LRDFIinv]) 
	return scalar sysgmmpcpfulladdLR = ttail(`dftest', -(_b[LRDFIinv]-1)/_se[LRDFIinv]) 

}
else if "`estimator'" == "IV" {

	/* FE IV regressions */
	xtivreg2 totalinv (dfiinvestment = DFIinvIV) ydummy* if year > 5, fe cluster(country) first savefirst small
	return scalar ivbeta =_b[dfiinvestment]
	return scalar ivse =_se[dfiinvestment]
	return scalar underid = e(idstat)
	return scalar underidp = e(idp)
	return scalar weakid = e(widstat)
	test dfiinvestment
	return scalar ivpzeroadd = ttail(r(df_r), _b[dfiinvestment]/_se[dfiinvestment]) 
	return scalar ivpfulladd = ttail(r(df_r), -(_b[dfiinvestment]-1)/_se[dfiinvestment]) 
	// pick up first stage coefficient
	xtreg dfiinvestment DFIinvIV ydummy* if year > 5, fe cluster(country) 
	return scalar ivfsbeta = _b[DFIinvIV]
	return scalar ivfsse = _se[DFIinvIV]
	/* Same but controlling for pc */
	xtivreg2 totalinv (dfiinvestment = DFIinvIV) pc ydummy* if year > 5, fe cluster(country) first savefirst small
	return scalar ivpcbeta =_b[dfiinvestment]
	return scalar ivpcse =_se[dfiinvestment]
	return scalar underidpc = e(idstat)
	return scalar underidppc = e(idp)
	return scalar weakidpc = e(widstat)
	test dfiinvestment
	return scalar ivpcpzeroadd = ttail(r(df_r), _b[dfiinvestment]/_se[dfiinvestment]) 
	return scalar ivpcpfulladd = ttail(r(df_r), -(_b[dfiinvestment]-1)/_se[dfiinvestment]) 
	// pick up first stage coefficient
	xtreg dfiinvestment DFIinvIV pc ydummy* if year > 5, fe cluster(country) 
	return scalar ivpcfsbeta = _b[DFIinvIV]
	return scalar ivpcfsse = _se[DFIinvIV]

}
else {
	di as error "Incorrect estimator specified"
}

end // end program



