* Paddy Carter and Nicolas Van de Sijpe, 15-July-2018
* Define program elusivefirms (a simplified version of elusive), used to generate firm-level data


/* This do file is called by firms final.do so we have commented out the preamble (the preamble is useful to do a single run to check the code works). 
The program elusivefirms is a simplified, stripped-down version of elusive. 
elusivefirms differs from elusive because it does not put the data in dyadic format or collapse it to country level, 
and we've changed the proportions of country types to 1/3 each (hard coded in the 'country types' section). 
The defaults for nC and t are smaller - these changes are just to produce clearer graphs. */


/*
clear all
set more off
set seed 7542629 // seed for a single run of the code below, to see whether program works



*************************************************************************
* GTOOLS	
*************************************************************************
/*
* We use gtools (version 0.13.1 02May2018) to speed up the MCs run by elusive: https://github.com/mcaceresb/stata-gtools
* To install:
local github "https://raw.githubusercontent.com"
net install gtools, from(`github'/mcaceresb/stata-gtools/master/build/)
* Note for Windows users: It may be necessary to run 
gtools, dependencies 
* at the start of your Stata session.
*/


*************************************************************************
*************************************************************************
* MONTE CARLO PROGRAM	
*************************************************************************
*************************************************************************
/* Program key: */
/* (nC num countries) (t years) (nD num DFIs) (phi DFI country preferences weight) (drift in budget random walk as % of initial budget db) 
(sig s.d. of dfi budget as % of initial budget db) (dfbudg t=1 DFI budget % of total ops) (tvweight weight on time-varying part of DFI preferences) 
(mu mean project characteristics of three country types) (musd s.d. for project characteristics of three country types) (ersd s.d. of expected returns added to pc)
(dfilo lower bound eligible dfi set) (dfihi upper bound dfi set) (psmin min exp return for private sector) 
(nI num inv opportunities per country-year) (noise error added to pc after collapse) estimation(estimator options are OLSFE, GMM, IV) 
(mechanism DFI inv selection mechanism: options are random, pclow)*/


capture program drop elusivefirms
program define elusivefirms, rclass
	version 15.1
	syntax [, nC(integer 10) t(integer 1) nD(real 3) phi(real 1) drift(real 0) sig(real 0.2) /// 
	dfbudg(real 0.1) tvweight(real 0) mu1(real 0) mu2(real 2) mu3(real 4) musd1(real 1) musd2(real 1) musd3(real 1) ersd(real 1) /// 
	dfilo(real 2) dfihi(real 8) psmin(real 2) nI(real 50) noise(real 0.5) estimator(string) mechanism(string)]

	
drop _all


/* Locals for single run of the program; otherwise commented out
local nC = 10
local t = 1
local nD = 3
local phi = 1
local drift = 0
local sig = 0.2
local dfbudg = 0.1
local tvweight = 0
local mu1 = 0
local mu2 = 2
local mu3 = 4
local musd1 = 1
local musd2 = 1
local musd3 = 1
local ersd = 1
local dfilo = 2
local dfihi = 8
local psmin = 2
local nI = 50
local noise = 0.5
local estimator OLSFE
local mechanism random
*/


* private sector budget (always exceeds available investment opportunities): 
local pb = `nI'*`nC'
* initial value for the total DFI budget in year 1; defined as a fraction (dfbudg) of maximum potential total number of annual investments:
local db = `dfbudg'*(`nI'*`nC')
* standard deviation for the random walk in the DFI budget process (defined as fraction sig of initial budget db):
local sigma = `sig'*`db'


/* Set up data: */	
* dataset:
local size = `nC'*`t'
set obs `size'
* number the countries:
gen country = ceil(_n/`t') 
* periods:
sort country	
by country: generate year = _n
xtset country year


**************************************************************************
* Generate weights to divide investments between multiple DFIs
**************************************************************************
* generate individual DFIs' country preferences that sum to 1 
forvalues j = 1/`nD' {
	sort year country
	gen double dfipref`j' = runiform() in 1/`nC' // generate across countries in first year only
	gegen double temp`j' = max(dfipref`j'), by(country) // this fills in each country's prefs across years
	drop dfipref`j'
	gen double tvtemp`j' = runiform() // time-varying part of DFI-country preferences.
	gen double dfipref`j' = (1-`tvweight')*temp`j' + `tvweight'*tvtemp`j'
	drop temp`j' tvtemp`j'
}
ds dfipref*
gegen double sum = rowtotal(`r(varlist)')
forvalues j = 1/`nD' {
	replace dfipref`j' = dfipref`j'/sum
}
drop sum


* generate individual DFIs' shares of each investment in each country 
* start with weighted preferences: weight = pref^phi
* turn into shares using share_i = weight_i / (sum weights) 
forvalues j = 1/`nD' {
	gen double dfiweight`j' = dfipref`j'^`phi'
}
ds dfiweight*
gegen double sum = rowtotal(`r(varlist)')
forvalues j = 1/`nD' {
	gen double dfishare`j' = dfiweight`j'/sum
}
drop sum


**************************************************************************
* Country types
**************************************************************************
/* Initial country types: */
* country types 1/3 each
gen type = 1
replace type = 2 if country>`nC'*(1/3)
replace type = 3 if country>`nC'*(2/3)


/* Evolution of country types over time: */
* generate a 0-1 random number to base probabilities on (also used elsewhere later on). 
generate double randnum = uniform()
/* We want to implement the following transition matrix: .85 & .10 & .05\\ .05 & .85 & .10\\ .05 & .05 & .9
Run transition matrix to change country types over time: */
sort country year
forvalues j = 2/`t' {
	// type 1 becoming type 2:
	replace type=2 if randnum>0.85 & randnum<=0.95 & L.type==1 & year==`j'
	// type 1 becoming type 3:
	replace type=3 if randnum>0.95 & L.type==1 & year==`j'
	
	// type 2 becoming type 1:
	replace type=1 if randnum<=0.05 & L.type==2 & year==`j'
	// type 2 becoming type 3
	replace type=3 if randnum>0.9 & L.type==2 & year==`j'
	
	// type 3 becoming type 1:
	replace type=1 if randnum<=0.05 & L.type==3 & year==`j'
	// type 3 becoming type 2:
	replace type=2 if randnum>0.05 & randnum<=0.1 & L.type==3 & year==`j'
}


**************************************************************************
* Budgets
**************************************************************************
/* Time-varying DFI budget. */
* a random walk with drift 
sort country year
local normdrift=`drift'*`db'
gen int db = `db' in 1/`t' // this just does the first country
* set budget so that it cannot fall below 20; and round so that db is always integer
replace db = round(max(20,L.db + `normdrift' + rnormal(0,`sigma'))) if year > 1 & db != .
gegen double db2 = max(db), by(year) // this fills in the same db for each year for the other countries
drop db
rename db2 db


/* Private budget. */
* Does not need to be stochastic, all that matters is whether pb is larger than bankable set of investments (as specified at the start of the code)
gen double pb = `pb'


**************************************************************************
* Individual projects
**************************************************************************
/* Data for individual projects */
expand `nI'
sort country year
* create an index for individual projects
gen i = _n


/* Project characteristics */
gen double pc = rnormal(`mu1',`musd1')
replace pc = rnormal(`mu2',`musd2') if type==2
replace pc = rnormal(`mu3',`musd3') if type==3
label var pc "Project characteristics"


/* Expected returns */
gen double er = pc + rnormal(0,`ersd')
label var er "Expected returns"


********************************************************************
* DFI investment selection mechanism
********************************************************************
if "`mechanism'" == "random" {
	/* DFI set of eligible projects is based on expected return. 
	Within this global set, in each year the DFI randomly picks projects until its budget runs out. */


	/* Selection of investments by DFI sector as a whole. */
	* Set of investments DFIs would like to invest in:
	gen dfiset = inrange(er,`dfilo',`dfihi')
	/* In each year, DFI sector picks investments from this global set of investment opportunities (so not within each county) until the budget is exhausted */
	/* Put investable projects in random order and pick from the top until the budget is exhausted, 
	so that DFI sector randomly picks projects from its global set of investment opportunities until it exhaust its budget. */
	replace randnum = uniform()
	sort year randnum // randomly sort projects within each year
	* cumulative count of number of projects in each year DFI is interested in:
	by year: gen cum_dfi = sum(dfiset) 
	* DFI sector invests if project is in eligible set and still within the budget:
	generate byte dfi_inv = dfiset == 1 & cum_dfi <= db 


	* Divide up these investments between multiple DFIs
	forvalues j = 1/`nD' {
		gen double dfi_inv_`j' = dfishare`j'*dfi_inv
	}


	/* Selection of investment by private investors: */
	gen psset = 0
	* DFI sector has first dibs so private sector can only pick from investments not already taken by DFI:
	replace psset = 1 if er > `psmin' & dfi_inv == 0
	* same trick to have private investors pick randomly from global set of projects they are interested in in each year
	sort year randnum
	by year: gen cum_ps = sum(psset)
	* private sector invests if the project is in the eligible set and still within the budget:
	generate byte ps_inv = psset == 1 & cum_ps <= pb

}
else if "`mechanism'" == "pclow" {

	/* DFI set of eligible projects is based on expected return. 
	Within this global set, in each year the DFI now first picks the projects with the worst project characteristics. */


	/* Selection of investments by DFI sector as a whole. */
	* Set of investments DFIs would like to invest in:
	gen dfiset = inrange(er,`dfilo',`dfihi')
	/* In each year, DFI sector picks investments from this global set of investment opportunities (so not within each county) until the budget is exhausted */
	/* Rank investment opportunities within each year by pc and pick lowest pc projects first, until budget is exhausted. */
	sort year pc
	* cumulative count of number of projects in each year DFI is interested in:
	by year: gen cum_dfi = sum(dfiset)
	* DFI invests if project is in eligible set and still within the budget:
	generate byte dfi_inv = dfiset == 1 & cum_dfi <= db


	* Divide up these investments between multiple DFIs
	forvalues j = 1/`nD' {
		gen double dfi_inv_`j' = dfishare`j'*dfi_inv
	}


	/* Selection of investment by private investors: */
	gen psset=0
	* DFI has first dibs so private sector can only pick from investments not already taken by DFI:
	replace psset = 1 if er > `psmin' & dfi_inv == 0
	* have private investors pick randomly from global set of projects they are interested in in each year
	replace randnum = uniform()
	sort year randnum
	by year: gen cum_ps = sum(psset)
	* private sector invests if the project is in the eligible set and still within the budget:
	generate byte ps_inv = psset == 1 & cum_ps <= pb 

}
else {
	di as error "Incorrect investment selection mechanism specified"
}

end 
